#! /usr/bin/env python3
# coding: utf8

import sys
from os import system
from math import pi, sin, cos, tan, atan, sqrt

# --- Definitions de classes

class Fichier:
    def __init__(self, nomFichierArg, boolEcriture):
        self.nomFichier = nomFichierArg
        self.justeApresInterruption = False
        if boolEcriture:
            self.fichier = open(self.nomFichier, 'wb') # 'b' pour des octets
            self.fichier.seek(0, 0) # Se place au debut du fichier
            self.longueur = 0
        else:
            self.fichier = open(self.nomFichier, 'rb') # 'b' pour des octets
            self.fichier.seek(0, 2) # Se place a la fin du fichier
            self.longueur = self.fichier.tell()
        self.index = 0

    def close (self):
        if not self.justeApresInterruption:
            pass
        self.fichier.close()
        self.justeApresInterruption = False

    def deByteStringAChaineUnicode (self, monByteString):
        chaineUnicode = monByteString.decode(encoding='UTF-8')
        return chaineUnicode

    def deChaineUnicodeAByteString (self, chaineUnicode):
        monByteString = chaineUnicode.encode(encoding='UTF-8')
        return monByteString

    def ecritUneLigne (self, ligneAEcrire):
        ligneAEcrire2 = self.deChaineUnicodeAByteString(ligneAEcrire)
        for numOctet in range(len(ligneAEcrire2)):
            octet = ligneAEcrire2[numOctet:numOctet + 1]
            # La ligne precedente a l'air amphigourique, mais
            # ligneAEcrire2[numOctet:numOctet + 1] est de type "bytes",
            # alors que ligneAEcrire2[numOctet] serait de type "int"
            self.ecritUnOctet(octet)
        # self.ecritUnOctet(b'\n') # Ecrit sous Windows comme b'\x0D' et non comme b'\x0A'
        self.ecritUnOctet(b'\x0A') # Encore à tester sous Windows

    def ecritUnOctet (self, signe):
        self.fichier.seek(self.index, 0)
        self.fichier.write(signe)
        self.index += 1
        self.longueur += 1

    def interromptLecture (self):
        self.justeApresInterruption = True
        self.close()

    def litUneLigne (self):
        octetLu = ''
        ligneLue = b"" # Soit un bytestring vide
        finDeLigne = False

        while self.index < self.longueur and not finDeLigne:
            octetLu = self.litUnOctet(self.index)
            if octetLu == b'\n':
                finDeLigne = True
            else:
                ligneLue += octetLu

        ligneLue2 = self.deByteStringAChaineUnicode(ligneLue)
        
        return ligneLue2

    def litUnOctet (self, numOctet):
        self.fichier.seek(numOctet, 0)
        octet = self.fichier.read(1)
        self.index += 1
        return octet

    def reprendLecture (self):
        self.fichier = open(self.nomFichier, 'rb')
        self.fichier.seek(self.index, 0) # Se place ou on s'etait arrete

    def seek (self, numOctet):
        self.fichier.seek(numOctet, 0)
        self.index = numOctet

# --- Definitions de fonctions

def principal (argv):
    nbArgs = len(sys.argv)

    if nbArgs >= 3:
        nomFichALire = sys.argv[1]
        nomFichAEcrire = sys.argv[2]
    else:
        vrcIni = litFichIni('3d_vers_3db.ini')
        nomFichALire = vrcIni['Source']
        nomFichAEcrire = vrcIni['Resultat']

    # I - Lecture de la hierarchie de fichiers pour constituer
    # un tableau unique (sans les instructions d'inclusion)
    mesLignesSansInclusion = []
    fichALire = Fichier(nomFichALire, False)
    litJusquaInclusion(fichALire, mesLignesSansInclusion)
    # fichALire.close() # litJusquaInclusion opère la fermeture
    
    # II - Intégration des modifications de coordonnées
    baliseEnCours = "rien"
    mesLignesModifiees = []
    
    nivHierModif = -1
    inventaireModifCoord = []
    indexDernModif = []
    nbLignesModifACeNiveau = []
    
    nbLignesSansInclusion = len(mesLignesSansInclusion)
    for numLigne in range(nbLignesSansInclusion):
        ligneLue = mesLignesSansInclusion[numLigne]
        ligneLue = chomp(ligneLue)
        if len(ligneLue) > 0 and ligneLue[0] == '<':
            # On a affaire a une ligne de balise
            if ligneLue[1] != '/':
                # On a affaire a une balise ouvrante
                baliseEnCours = ligneLue[1:len(ligneLue) - 1]
                if baliseEnCours == "modifcoord":
                    nivHierModif += 1
                    nbLignesModifACeNiveau.append(0)
                    nbModifCoordACeStade = len(inventaireModifCoord)
                    indexDernModif.append(nbModifCoordACeStade - 1)
                    # C'est l'index de la dernière modif, pas le nombre de modif,
                    # et ca doit être inférieur de 1 au premier indice effectif,
                    # puisque la lecture de chaque modif entraîne un incrément.
                elif baliseEnCours == "points3D":
                    listeModifOrdonnee = \
                        ecritlisteModifPertinente(inventaireModifCoord, nivHierModif, \
                                                  indexDernModif)
                    mesLignesModifiees.append(ligneLue)
                else:
                    mesLignesModifiees.append(ligneLue)
            else:
                # On a affaire a une balise fermante
                baliseEnCours = "rien"
                baliseFermee = ligneLue[2:len(ligneLue) - 1]
                if baliseFermee != "modifcoord":
                    mesLignesModifiees.append(ligneLue)
        else:
            # On a affaire a une ligne d'instructions
            if baliseEnCours == 'modifcoord':
                if ligneLue != "Fin":
                    nbLignesModifACeNiveau[nivHierModif] += 1
                    inventaireModifCoord.append(ligneLue)
                    indexDernModif[nivHierModif] += 1
                else:
                    # La balise <modifcoord> provoque elle-même un incrément,
                    # que l'on neutralise tout de suite
                    nivHierModif -= 1
                    indexDernModif.pop()
                    
                    ancienNiveauHierModif = nivHierModif
                    
                    (inventaireModifCoord, nivHierModif, indexDernModif) = \
                        popUnNiveauModif(inventaireModifCoord, nivHierModif, indexDernModif)
                    # Je rappelle que quand aucune modifcoord n'est en vigueur,
                    # le niveau est -1 et non 0.
            elif baliseEnCours == 'points3D':
                mesBazars = ligneLue.split(',')
                numPoint = int(mesBazars[0])
                xrays = float(mesBazars[1])
                yankee = float(mesBazars[2])
                zulu = float(mesBazars[3])
                monPoint = (numPoint, xrays, yankee, zulu)
                
                for modif in listeModifOrdonnee:
                    typeModif = modif[0:4]
                    if typeModif == "Posi":
                        monPoint = modifPosi(monPoint, modif)
                    elif typeModif == "Tail":
                        monPoint = modifTaille(monPoint, modif)
                    elif typeModif == "Rotx":
                        monPoint = rotX(monPoint, modif)
                    elif typeModif == "Roty":
                        monPoint = rotY(monMoint, modif)
                    elif typeModif == "Rotz":
                        monPoint = rotZ(monPoint, modif)
                    else:
                        print("*** La ligne de modif [%s] n'a pas la bonne syntaxe." % (modif))
                        sys.exit(1)
                    
                (numPoint, xrays, yankee, zulu) = monPoint
                ligneModifiee = "%d,%f,%f,%f" % (numPoint, xrays, yankee, zulu)
                mesLignesModifiees.append(ligneModifiee)
            else:
                mesLignesModifiees.append(ligneLue)

    # III - Recopiage des lignes sans balises modifcoord mais avec les coordonnées modifiées
    fichAEcrire = Fichier(nomFichAEcrire, True)
    nbLignesModifiees = len(mesLignesModifiees)
    for numLigne in range(nbLignesModifiees):
        maLigne = mesLignesModifiees[numLigne]
        fichAEcrire.ecritUneLigne(maLigne)
    fichAEcrire.close()

def chomp (maChaine):
# remplace la fonction du meme nom en Perl, qui vire le ou les
# caracteres de fin de ligne
    index = 0
    maChaine2 = ''
    while index < len(maChaine):
        signe = maChaine[index]
        if signe != '\n' and signe != '\r':
            maChaine2 += signe
        index += 1
    return maChaine2

def convPolRec (direction, distance):
    deltaX = cos(direction) * distance
    deltaY = sin(direction) * distance

    return (deltaX, deltaY)

def convRecPol (deltaX, deltaY):
    distance = sqrt(deltaX * deltaX + deltaY * deltaY)
    if abs(deltaX) > abs(deltaY):
    # Pour eviter les risques de division par 0 ou quasi
        direction = atan(deltaY / deltaX)
        if deltaX < 0:
            direction += pi
    else:
        direction = atan(deltaX / deltaY)
        if deltaY < 0:
            direction += pi
        direction = pi / 2.0 - direction
    direction = nrmlzDir(direction)

    return (direction, distance)

def ecritlisteModifPertinente (monInventaire, nivHierModif, indexDernModif):
    nbNiveauxModif = nivHierModif + 1
    
    listeModifOrdonnee = []

    for numNiveau in range(nbNiveauxModif - 1, -1, -1):
        # En itération inverse des niveaux parce qu'il faut intégrer
        # les modifications en partant du niveau hiérarchique le plus
        # profond pour remonter graduellement vers le niveau englobant
        if numNiveau == 0:
            numPremiereLigneModif = 0
        else:
            numPremiereLigneModif = indexDernModif[numNiveau - 1] + 1
        if numNiveau == nbNiveauxModif - 1:
            numDerniereLigneModif = len(monInventaire) - 1
        else:
            numDerniereLigneModif = indexDernModif[numNiveau]
        for numLigne in range(numPremiereLigneModif, numDerniereLigneModif + 1):
            listeModifOrdonnee.append(monInventaire[numLigne])
            
    return listeModifOrdonnee

def litFichIni (nomFichIni):
    vrcIni = {}
    
    # Lecture du fichier d'initialisation et stockage de toutes
    # les valeurs dans le dictionnaire, mais sous la forme de chaines
    fichierSource = open(nomFichIni, 'r')
    ligneLue = 'rien'
    while ligneLue != '':
        ligneLue = fichierSource.readline()
        if ligneLue != '':
            ligneLue2 = chomp(ligneLue)
            if yAUnSigne('=', ligneLue2):
                tableau = ligneLue2.split('=')
                cle = tableau[0].strip()
                valeur = tableau[1].strip()
                vrcIni[cle] = valeur
    fichierSource.close()
    
    # Affectation du type correct a chacune des valeurs du dictionnaire
    cles = vrcIni.keys()
    for cle in cles:
        if typeString(vrcIni[cle]) == 1:
            vrcIni[cle] = float(vrcIni[cle])
        elif typeString(vrcIni[cle]) == 2:
            vrcIni[cle] = int(vrcIni[cle])
        elif vrcIni[cle].lower() == 'true':
            vrcIni[cle] = True
        elif vrcIni[cle].lower() == 'false':
            vrcIni[cle] = False

    return vrcIni

def litJusquaInclusion (fichALire, mesLignesSansInclusion):
    boolBlzInclLgnPrec = False

    while fichALire.index < fichALire.longueur:
        ligneLue = fichALire.litUneLigne()
        if ligneLue == '<inclusion>':
            boolBlzInclLgnPrec = True
        else:
            if boolBlzInclLgnPrec:
                fichALire.interromptLecture() # Evite d'avoir trop de fichiers ouverts
                nouvFichALire = Fichier(ligneLue, False)
                litJusquaInclusion(nouvFichALire, mesLignesSansInclusion) # Appel recursif
                fichALire.reprendLecture()
            else:
                if ligneLue != '<inclusion>' and ligneLue != '</inclusion>':
                    mesLignesSansInclusion.append(ligneLue)
            boolBlzInclLgnPrec = False

    fichALire.close()

def modifPosi (monPoint, modif):
    (numPoint, xrays, yankee, zulu) = monPoint
    
    valeurModifString = modif[5:]
    mesBazars = valeurModifString.split(',')
    rabXrays = float(mesBazars[0])
    rabYankee = float(mesBazars[1])
    rabZulu = float(mesBazars[2])
    
    xrays += rabXrays
    yankee += rabYankee
    zulu += rabZulu
    
    monPoint = (numPoint, xrays, yankee, zulu)
    
    return monPoint
    
def modifTaille (monPoint, modif):
    (numPoint, xrays, yankee, zulu) = monPoint
    
    valeurModifString = modif[5:]
    mesBazars = valeurModifString.split(',')
    multXrays = float(mesBazars[0])
    multYankee = float(mesBazars[1])
    multZulu = float(mesBazars[2])
    
    xrays *= multXrays
    yankee *= multYankee
    zulu *= multZulu
    
    monPoint = (numPoint, xrays, yankee, zulu)
    
    return monPoint

def nrmlzDir (direction):
    while direction > pi:
        direction -= 2.0 * pi
    while direction < pi * -1.0:
        direction += 2.0 * pi

    return direction

def popUnNiveauModif (monInventaire, nivHierModif, indexDernModif):
    nbNiveauxModif = nivHierModif + 1

    numDernierNiveau = nivHierModif
    if numDernierNiveau == 0:
        numPremiereLigneModif = 0
    else:
        numPremiereLigneModif = indexDernModif[numDernierNiveau - 1] + 1

    for numModif in range(len(monInventaire) - 1, numPremiereLigneModif - 1, -1):
        monInventaire.pop()
    indexDernModif.pop()

    nivHierModif -= 1

    return(monInventaire, nivHierModif, indexDernModif)

def rotation (abscisse, ordonnee, angleRad):
    (direction, distance) = convRecPol(abscisse, ordonnee)
    direction += angleRad
    direction = nrmlzDir(direction)
    (abscisse, ordonnee) = convPolRec(direction, distance)
    
    return (abscisse, ordonnee)

def rotX (monPoint, modif):
    (numPoint, xrays, yankee, zulu) = monPoint
    
    angleDeg = float(modif[5:])
    angleRad = angleDeg * pi / 180.0
    
    (yankee, zulu) = rotation(yankee, zulu, angleRad)
    
    monPoint = (numPoint, xrays, yankee, zulu)

    return monPoint

def rotY (monPoint, modif):
    (numPoint, xrays, yankee, zulu) = monPoint
    
    angleDeg = float(modif[5:])
    angleRad = angleDeg * pi / 180.0
    
    (zulu, xrays) = rotation(zulu, xrays, angleRad)
    
    monPoint = (numPoint, xrays, yankee, zulu)

    return monPoint

def rotZ (monPoint, modif):
    (numPoint, xrays, yankee, zulu) = monPoint
    
    angleDeg = float(modif[5:])
    angleRad = angleDeg * pi / 180.0
    
    (xrays, yankee) = rotation(xrays, yankee, angleRad)
    
    monPoint = (numPoint, xrays, yankee, zulu)

    return monPoint

def typeString (maChaine):
# Python etant type, il faut pouvoir determiner le type
# d'une variable enregistree en Ascii dans le fichier
# d'initialisation. Cette routine reconnait les nombres,
# distingue entre entiers et nombres a point decimal, et
# considere tout le reste comme des chaines
    index = 0
    boolNumerique = True
    while index < len(maChaine):
        signe = maChaine[index]
        if not (signe == '.' or signe == '-' or \
                (signe >= '0' and signe <= '9')):
            boolNumerique = False
        index += 1

    boolEntier = True
    if boolNumerique:
        index = 0
        while index < len(maChaine):
            signe = maChaine[index]
            if signe == '.':
                boolEntier = False
            index += 1

    typeDeChaine = boolNumerique
    if boolNumerique and boolEntier:
        typeDeChaine = 2

    return typeDeChaine # 0 = str, 1 = float, 2 = int

def yAUnSigne (signeAChercher, maChaine):
# S'assure de la presence d'un signe precis dans une ligne.
# Sert notamment a reperer le signe = dans les lignes d'un
# fichier d'initialisation
    boolYAUnSigne = False
    index = 0
    while index < len(maChaine):
        signe = maChaine[index]
        if signe == signeAChercher:
            boolYAUnSigne = True
        index += 1
    return boolYAUnSigne

# --- Fin des definitions de classes et de fonctions,
# --- debut du programme principal

principal(sys.argv)
