#! /usr/bin/env python3
# coding: utf8

# Convertisseur Alias-Wavefront -> SGML 3D plus ambitieux que
# la version précédente dont il s'inspire: quand le fichier .obj
# est structuré en objets indépendants, cette moulinette produit
# des fichiers 3D indépendants, avec renumérotation des points.

import sys
from os import system

# --- Definitions de classes

class Point3D:
    def __init__(self, xraysArg, yankeeArg, zuluArg):
        self.xrays = xraysArg
        self.yankee = yankeeArg
        self.zulu = zuluArg
        self.appele = False # par defaut
        self.nuvoNum = -1

class Fichier:
    def __init__(self, nomFichierArg, boolEcriture):
        self.nomFichier = nomFichierArg
        if boolEcriture:
            self.fichier = open(self.nomFichier, 'wb') # 'b' pour des octets
            self.fichier.seek(0, 0) # Se place au debut du fichier
            self.longueur = 0
        else:
            self.fichier = open(self.nomFichier, 'rb') # 'b' pour des octets
            self.fichier.seek(0, 2) # Se place a la fin du fichier
            self.longueur = self.fichier.tell()
        self.index = 0

    def close (self):
        self.fichier.close()

    def deByteStringAChaineUnicode (self, monByteString):
        chaineUnicode = monByteString.decode(encoding='UTF-8')
        return chaineUnicode

    def deChaineUnicodeAByteString (self, chaineUnicode):
        monByteString = chaineUnicode.encode(encoding='UTF-8')
        return monByteString

    def ecritUneLigne (self, ligneAEcrire):
        ligneAEcrire2 = self.deChaineUnicodeAByteString(ligneAEcrire)
        for numOctet in range(len(ligneAEcrire2)):
            octet = ligneAEcrire2[numOctet:numOctet + 1]
            # La ligne precedente a l'air amphigourique, mais
            # ligneAEcrire2[numOctet:numOctet + 1] est de type "bytes",
            # alors que ligneAEcrire2[numOctet] serait de type "int"
            self.ecritUnOctet(octet)
        # self.ecritUnOctet(b'\n') # Ecrit sous Windows comme b'\x0D' et non comme b'\x0A'
        self.ecritUnOctet(b'\x0A') # Encore à tester sous Windows

    def ecritUnOctet (self, signe):
        self.fichier.seek(self.index, 0)
        self.fichier.write(signe)
        self.index += 1
        self.longueur += 1

    def interromptLecture (self):
        self.close()

    def litUneLigne (self):
        octetLu = ''
        ligneLue = b"" # Soit un bytestring vide
        finDeLigne = 0 # false

        while self.index < self.longueur and not finDeLigne:
            octetLu = self.litUnOctet(self.index)
            if octetLu == b'\n':
                finDeLigne = 1 # true
            else:
                ligneLue += octetLu

        ligneLue2 = self.deByteStringAChaineUnicode(ligneLue)
        return ligneLue2

    def litUnOctet (self, numOctet):
        self.fichier.seek(numOctet, 0)
        octet = self.fichier.read(1)
        self.index += 1
        return octet

    def reprendLecture (self):
        self.fichier = open(self.nomFichier, 'r')
        self.fichier.seek(self.index, 0) # Se place ou on s'etait arrete

    def seek (self, numOctet):
        self.fichier.seek(numOctet, 0)
        self.index = numOctet

# --- Definitions de fonctions

def principal (argv):
    nbArgs = len(sys.argv)

    if nbArgs >= 3:
        nomFichALire = sys.argv[1]
        nomFichAEcrire = sys.argv[2]
    else:
        print("*** Il faut donner en arg. le nom des fichiers lu et ecrit.")
        sys.exit(1)


    # D'abord, lecture et stockage en mémoire de tous les points
    mesPoints = []
    fichALire = Fichier(nomFichALire, 0)
    
    numPoint = 1
    while fichALire.index < fichALire.longueur:
        ligneLue = fichALire.litUneLigne()
        if len(ligneLue) >= 2 and ligneLue[0:2] == "v ":
            mesBazars = ligneLue.split(' ')
            xrays = float(mesBazars[1])
            yankee = float(mesBazars[3]) * -1.0
            zulu = float(mesBazars[2])
            monPoint = Point3D(xrays, yankee, zulu)
            mesPoints.append(monPoint)
            numPoint += 1
    fichALire.close()

#    ligneAEcrire = "<points3D>"
#    fichAEcrire.ecritUneLigne(ligneAEcrire)
#    ligneAEcrire = "</points3D>"
#    fichAEcrire.ecritUneLigne(ligneAEcrire)

    # Ensuite, lecture des faces et constitution des objets
    mesObjets = []
    mesFaces = []
    fichALire = Fichier(nomFichALire, 0)
    fichEnEcritureOuvert = False
    
    while fichALire.index < fichALire.longueur:
        ligneLue = fichALire.litUneLigne()
        if len(ligneLue) >= 2 and ligneLue[0:2] == "o ":
            mesBazars = ligneLue.split(' ')
            nomObjet = mesBazars[1]
            mesObjets.append(nomObjet)
            if fichEnEcritureOuvert:
                fichEnEcritureOuvert = stockeObjet(mesPoints, mesFaces, fichAEcrire)
            fichAEcrire = Fichier(("%s.3d" % (nomObjet)), 1)
            fichEnEcritureOuvert = True
            mesFaces = []
        elif len(ligneLue) >= 2 and ligneLue[0:2] == "f ":
            mesBazars = ligneLue.split(' ')
            maFace = ""
            for numBazar in range(len(mesBazars) - 1):
                mesBazarsBis = (mesBazars[numBazar + 1]).split('/')
                monNumPoint = int(mesBazarsBis[0])
                maFace += "%d" % (monNumPoint)
                if numBazar < len(mesBazars) - 2:
                    maFace += ","
            mesFaces.append(maFace)
    fichALire.close()
    if fichEnEcritureOuvert:
        fichEnEcritureOuvert = stockeObjet(mesPoints, mesFaces, fichAEcrire)

    # Ecriture d'un fichier source global
    fichAEcrire = Fichier(nomFichAEcrire, 1)
    for numObjet in range(len(mesObjets)):
        nomObjet = mesObjets[numObjet]
        fichAEcrire.ecritUneLigne("<inclusion>")
        ligneAEcrire = "%s.3d" % (nomObjet)
        fichAEcrire.ecritUneLigne(ligneAEcrire)
        fichAEcrire.ecritUneLigne("</inclusion>")
    fichAEcrire.close()

def stockeObjet(mesPoints, mesFaces, fichAEcrire):
    # On purge les conséquences des traitements des objets précédents
    for numPoint in range(len(mesPoints)):
        mesPoints[numPoint].appele = False
        mesPoints[numPoint].nuvoNum = -1

    # On note quels points sont appeles
    for numFace in range(len(mesFaces)):
        maFace = mesFaces[numFace]
        mesBazars = maFace.split(',')
        for numBazar in range(len(mesBazars)):
            numPoint = int(mesBazars[numBazar]) - 1 # Le tableau est numerote
                                                    # a partir de 0 et non de 1
            mesPoints[numPoint].appele = True

    # On donne de nouveaux numéros aux points appelés et on les stocke
    ligneAEcrire = "<points3D>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    plusHautNuvoNum = 1 # Là, on commence la numérotation à 1
    for numPoint in range(len(mesPoints)):
        if mesPoints[numPoint].appele:
            mesPoints[numPoint].nuvoNum = plusHautNuvoNum
            plusHautNuvoNum += 1
            ligneAEcrire = "%d,%f,%f,%f" % (mesPoints[numPoint].nuvoNum, \
                                            mesPoints[numPoint].xrays, \
                                            mesPoints[numPoint].yankee, \
                                            mesPoints[numPoint].zulu)
            fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = "</points3D>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    
    # On stocke les faces
    ligneAEcrire = "<faces>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    for numFace in range(len(mesFaces)):
        maFace = mesFaces[numFace]
        mesBazars = maFace.split(',')
        ligneAEcrire = ""
        for numBazar in range(len(mesBazars)):
            monNum = mesPoints[int(mesBazars[numBazar]) - 1].nuvoNum
            ligneAEcrire += "%d" % (monNum)
            if numBazar < len(mesBazars) - 1:
                ligneAEcrire += ","
        fichAEcrire.ecritUneLigne(ligneAEcrire)
    ligneAEcrire = "</faces>"
    fichAEcrire.ecritUneLigne(ligneAEcrire)
    fichAEcrire.close()
    fichEnEcritureOuvert = False

    return fichEnEcritureOuvert

# --- Fin des definitions de classes et de fonctions,
# --- debut du programme principal

principal(sys.argv)
