#! /usr/bin/env python3
# coding: utf8

import sys
from os import system
from math import pi, sin, cos, tan, atan, sqrt

# Ce listing n'est comprehensible que si on l'accompagne
# du schema variables_mur_sud.png... mais alors il se révèle
# très simple.
#
# Les dimensions sont généralement décalquées (avec quelques
# adaptations minimes) des notes prises dans le fichier
# pallud_dimensions.org

# --- Definitions de classes

class Fichier:
    def __init__(self, nomFichierArg, boolEcriture):
        self.nomFichier = nomFichierArg
        if boolEcriture:
            self.fichier = open(self.nomFichier, 'wb') # 'b' pour des octets
            self.fichier.seek(0, 0) # Se place au debut du fichier
            self.longueur = 0
        else:
            self.fichier = open(self.nomFichier, 'rb') # 'b' pour des octets
            self.fichier.seek(0, 2) # Se place a la fin du fichier
            self.longueur = self.fichier.tell()
        self.index = 0

    def close (self):
        self.fichier.close()

    def deByteStringAChaineUnicode (self, monByteString):
        chaineUnicode = monByteString.decode(encoding='UTF-8')
        return chaineUnicode

    def deChaineUnicodeAByteString (self, chaineUnicode):
        monByteString = chaineUnicode.encode(encoding='UTF-8')
        return monByteString

    def ecritUneLigne (self, ligneAEcrire):
        ligneAEcrire2 = self.deChaineUnicodeAByteString(ligneAEcrire)
        for numOctet in range(len(ligneAEcrire2)):
            octet = ligneAEcrire2[numOctet:numOctet + 1]
            # La ligne precedente a l'air amphigourique, mais
            # ligneAEcrire2[numOctet:numOctet + 1] est de type "bytes",
            # alors que ligneAEcrire2[numOctet] serait de type "int"
            self.ecritUnOctet(octet)
        # self.ecritUnOctet(b'\n') # Ecrit sous Windows comme b'\x0D' et non comme b'\x0A'
        self.ecritUnOctet(b'\x0A') # Encore à tester sous Windows

    def ecritUnOctet (self, signe):
        self.fichier.seek(self.index, 0)
        self.fichier.write(signe)
        self.index += 1
        self.longueur += 1

    def interromptLecture (self):
        self.close()

    def litUneLigne (self):
        octetLu = ''
        ligneLue = b"" # Soit un bytestring vide
        finDeLigne = 0 # false

        while self.index < self.longueur and not finDeLigne:
            octetLu = self.litUnOctet(self.index)
            if octetLu == b'\n':
                finDeLigne = 1 # true
            else:
                ligneLue += octetLu

        ligneLue2 = self.deByteStringAChaineUnicode(ligneLue)
        return ligneLue2

    def litUnOctet (self, numOctet):
        self.fichier.seek(numOctet, 0)
        octet = self.fichier.read(1)
        self.index += 1
        return octet

    def reprendLecture (self):
        self.fichier = open(self.nomFichier, 'rb') # 'b' pour des octets
        self.fichier.seek(self.index, 0) # Se place ou on s'etait arrete

    def seek (self, numOctet):
        self.fichier.seek(numOctet, 0)
        self.index = numOctet

# --- Definitions de fonctions

def principal (argv):
    nbArgs = len(sys.argv)

    if nbArgs >= 2:
        nomFichAEcrire = sys.argv[1]
    else:
        print("*** Il faut donner en arg. le nom du fichier ecrit.")
        sys.exit(1)

    fichAEcrire = Fichier(nomFichAEcrire, 1)

    largeurMaison = 8.99
    hauteurMaison = 9.28 # Estimation, certainement à revoir
    
    hauteurFenetre = 1.655
    hauteurEtage0 = 2.86
    hauteurEtage1 = 2.625
    largeurPorte = 1.0
    largeurFenetre = 0.9
    entreFenetres = 1.51 # En fait, entre une fenetre latérale et la porte ou
                         # la porte-fenêtre centrale
    entreEtages = 0.25 # Pifomètre
    sousFenetre = 0.8 # Hélas non mesuré, mais le bas de la menuiserie des panneaux
                      # de la fenêtre de la salle à manger était exactement au niveau
                      # du plateau de la table... et il faut tenir compte de quelques
                      # centimetres de menuiserie du cadre de la fenêtre par rapport
                      # au support de granit

    entreFenetreEtPlafondEtage0 = hauteurEtage0 - sousFenetre - hauteurFenetre
    entreFenetreEtPlafondEtage1 = hauteurEtage1 - sousFenetre - hauteurFenetre
    
    inclinaisonToitDeg = 30.0 # Pifomètre
    inclinaisonToitRad = inclinaisonToitDeg * pi / 180.0

    hauteurToit = (largeurMaison / 2) * tan(inclinaisonToitRad)
    hauteurDeuxEtages = hauteurEtage0 + entreEtages + hauteurEtage1

    # Les deux éléments qui suivent sont calculés de façon à ce que
    # tout tombe juste.
    deFenetreAMur = (largeurMaison - 2 * largeurFenetre - 2 * entreFenetres - largeurPorte) / 2
    sousRezDeChaussee = hauteurMaison - hauteurToit - hauteurDeuxEtages
    
    # Calcul des cotes indiquées sur le schéma
    alfa = 0.0
    bravo = alfa + sousRezDeChaussee
    charlie = bravo + sousFenetre
    delta = charlie + hauteurFenetre
    echo = delta + entreFenetreEtPlafondEtage0
    foxtrot = echo + entreEtages
    golf = foxtrot + sousFenetre
    hotel = golf + hauteurFenetre
    india = hotel + entreFenetreEtPlafondEtage1 + entreEtages
    juliet = india + hauteurToit
    altMidIndiaJuliet = india + hauteurToit / 2
    
    kilo = 0.0 - (largeurMaison / 2)
    lima = kilo + deFenetreAMur
    mike = lima + largeurFenetre
    mikePrime = 0.0 - largeurMaison / 4 # Milieu d'un pan de toit
    november = mike + entreFenetres
    oscar = november + largeurPorte
    papa = oscar + entreFenetres
    papaPrime = largeurMaison / 4 # Milieu d'un pan de toit
    quebec = papa + largeurFenetre
    romeo = largeurMaison / 2
    
    fichAEcrire.ecritUneLigne("<points3D>")
    
    # Recto
    ecritPointY0(1, kilo, alfa, fichAEcrire)
    ecritPointY0(2, mike, alfa, fichAEcrire)
    ecritPointY0(3, 0.0, alfa, fichAEcrire)
    ecritPointY0(4, papa, alfa, fichAEcrire)
    ecritPointY0(5, romeo, alfa, fichAEcrire)
    
    ecritPointY0(6, lima, charlie, fichAEcrire)
    ecritPointY0(7, mike, charlie, fichAEcrire)
    ecritPointY0(8, november, bravo, fichAEcrire)
    ecritPointY0(9, oscar, bravo, fichAEcrire)
    ecritPointY0(10, papa, charlie, fichAEcrire)
    ecritPointY0(11, quebec, charlie, fichAEcrire)
    
    ecritPointY0(12, lima, delta, fichAEcrire)
    ecritPointY0(13, mike, delta, fichAEcrire)
    ecritPointY0(14, november, delta, fichAEcrire)
    ecritPointY0(15, oscar, delta, fichAEcrire)
    ecritPointY0(16, papa, delta, fichAEcrire)
    ecritPointY0(17, quebec, delta, fichAEcrire)
    
    ecritPointY0(18, kilo, echo, fichAEcrire)
    ecritPointY0(19, mike, echo, fichAEcrire)
    ecritPointY0(20, 0.0, echo, fichAEcrire)
    ecritPointY0(21, papa, echo, fichAEcrire)
    ecritPointY0(22, romeo, echo, fichAEcrire)
    
    ecritPointY0(23, lima, golf, fichAEcrire)
    ecritPointY0(24, mike, golf, fichAEcrire)
    ecritPointY0(25, november, foxtrot, fichAEcrire)
    ecritPointY0(26, oscar, foxtrot, fichAEcrire)
    ecritPointY0(27, papa, golf, fichAEcrire)
    ecritPointY0(28, quebec, golf, fichAEcrire)
    
    ecritPointY0(29, lima, hotel, fichAEcrire)
    ecritPointY0(30, mike, hotel, fichAEcrire)
    ecritPointY0(31, november, hotel, fichAEcrire)
    ecritPointY0(32, oscar, hotel, fichAEcrire)
    ecritPointY0(33, papa, hotel, fichAEcrire)
    ecritPointY0(34, quebec, hotel, fichAEcrire)
    
    ecritPointY0(35, kilo, india, fichAEcrire)
    ecritPointY0(36, mike, india, fichAEcrire)
    ecritPointY0(37, 0.0, india, fichAEcrire)
    ecritPointY0(38, papa, india, fichAEcrire)
    ecritPointY0(39, romeo, india, fichAEcrire)
    
    ecritPointY0(40, mikePrime, altMidIndiaJuliet, fichAEcrire)
    ecritPointY0(41, papaPrime, altMidIndiaJuliet, fichAEcrire)
    ecritPointY0(42, 0.0, juliet, fichAEcrire)
    
    # Verso
    ecritPointY1(1, kilo, alfa, fichAEcrire)
    ecritPointY1(2, mike, alfa, fichAEcrire)
    ecritPointY1(3, 0.0, alfa, fichAEcrire)
    ecritPointY1(4, papa, alfa, fichAEcrire)
    ecritPointY1(5, romeo, alfa, fichAEcrire)
    
    ecritPointY1(6, lima, charlie, fichAEcrire)
    ecritPointY1(7, mike, charlie, fichAEcrire)
    ecritPointY1(8, november, bravo, fichAEcrire)
    ecritPointY1(9, oscar, bravo, fichAEcrire)
    ecritPointY1(10, papa, charlie, fichAEcrire)
    ecritPointY1(11, quebec, charlie, fichAEcrire)
    
    ecritPointY1(12, lima, delta, fichAEcrire)
    ecritPointY1(13, mike, delta, fichAEcrire)
    ecritPointY1(14, november, delta, fichAEcrire)
    ecritPointY1(15, oscar, delta, fichAEcrire)
    ecritPointY1(16, papa, delta, fichAEcrire)
    ecritPointY1(17, quebec, delta, fichAEcrire)
    
    ecritPointY1(18, kilo, echo, fichAEcrire)
    ecritPointY1(19, mike, echo, fichAEcrire)
    ecritPointY1(20, 0.0, echo, fichAEcrire)
    ecritPointY1(21, papa, echo, fichAEcrire)
    ecritPointY1(22, romeo, echo, fichAEcrire)
    
    ecritPointY1(23, lima, golf, fichAEcrire)
    ecritPointY1(24, mike, golf, fichAEcrire)
    ecritPointY1(25, november, foxtrot, fichAEcrire)
    ecritPointY1(26, oscar, foxtrot, fichAEcrire)
    ecritPointY1(27, papa, golf, fichAEcrire)
    ecritPointY1(28, quebec, golf, fichAEcrire)
    
    ecritPointY1(29, lima, hotel, fichAEcrire)
    ecritPointY1(30, mike, hotel, fichAEcrire)
    ecritPointY1(31, november, hotel, fichAEcrire)
    ecritPointY1(32, oscar, hotel, fichAEcrire)
    ecritPointY1(33, papa, hotel, fichAEcrire)
    ecritPointY1(34, quebec, hotel, fichAEcrire)
    
    ecritPointY1(35, kilo, india, fichAEcrire)
    ecritPointY1(36, mike, india, fichAEcrire)
    ecritPointY1(37, 0.0, india, fichAEcrire)
    ecritPointY1(38, papa, india, fichAEcrire)
    ecritPointY1(39, romeo, india, fichAEcrire)
    
    ecritPointY1(40, mikePrime, altMidIndiaJuliet, fichAEcrire)
    ecritPointY1(41, papaPrime, altMidIndiaJuliet, fichAEcrire)
    ecritPointY1(42, 0.0, juliet, fichAEcrire)
    
    fichAEcrire.ecritUneLigne("</points3D>")
    
    fichAEcrire.ecritUneLigne("<faces>")

    # Recto
    fichAEcrire.ecritUneLigne("1,2,7,6")
    fichAEcrire.ecritUneLigne("2,3,8,7")
    fichAEcrire.ecritUneLigne("3,9,8")
    fichAEcrire.ecritUneLigne("3,4,10,9")
    fichAEcrire.ecritUneLigne("4,5,11,10")
    fichAEcrire.ecritUneLigne("1,6,12,18")
    fichAEcrire.ecritUneLigne("7,8,14,13")
    fichAEcrire.ecritUneLigne("9,10,16,15")
    fichAEcrire.ecritUneLigne("11,5,22,17")
    fichAEcrire.ecritUneLigne("12,13,19,18")
    fichAEcrire.ecritUneLigne("13,14,20,19")
    fichAEcrire.ecritUneLigne("14,15,20")
    fichAEcrire.ecritUneLigne("15,16,21,20")
    fichAEcrire.ecritUneLigne("16,17,22,21")
    fichAEcrire.ecritUneLigne("18,19,24,23")
    fichAEcrire.ecritUneLigne("19,20,25,24")
    fichAEcrire.ecritUneLigne("20,26,25")
    fichAEcrire.ecritUneLigne("20,21,27,26")
    fichAEcrire.ecritUneLigne("21,22,28,27")
    fichAEcrire.ecritUneLigne("18,23,29,35")
    fichAEcrire.ecritUneLigne("24,25,31,30")
    fichAEcrire.ecritUneLigne("26,27,33,32")
    fichAEcrire.ecritUneLigne("28,22,39,34")
    fichAEcrire.ecritUneLigne("29,30,36,35")
    fichAEcrire.ecritUneLigne("30,31,37,36")
    fichAEcrire.ecritUneLigne("31,32,37")
    fichAEcrire.ecritUneLigne("32,33,38,37")
    fichAEcrire.ecritUneLigne("33,34,39,38")
    fichAEcrire.ecritUneLigne("35,36,40")
    fichAEcrire.ecritUneLigne("36,37,42,40")
    fichAEcrire.ecritUneLigne("37,38,41,42")
    fichAEcrire.ecritUneLigne("38,39,41")

    # Entourage fenêtres
    fichAEcrire.ecritUneLigne("6,7,49,48")
    fichAEcrire.ecritUneLigne("7,13,55,49")
    fichAEcrire.ecritUneLigne("55,13,12,54")
    fichAEcrire.ecritUneLigne("6,48,54,12")
    fichAEcrire.ecritUneLigne("8,9,51,50")
    fichAEcrire.ecritUneLigne("51,9,15,57")
    fichAEcrire.ecritUneLigne("14,56,57,15")
    fichAEcrire.ecritUneLigne("8,50,56,14")
    fichAEcrire.ecritUneLigne("10,11,53,52")
    fichAEcrire.ecritUneLigne("53,11,17,59")
    fichAEcrire.ecritUneLigne("58,59,17,16")
    fichAEcrire.ecritUneLigne("10,52,58,16")
    fichAEcrire.ecritUneLigne("27,28,70,69")
    fichAEcrire.ecritUneLigne("70,28,34,76")
    fichAEcrire.ecritUneLigne("76,34,33,75")
    fichAEcrire.ecritUneLigne("27,69,75,33")
    fichAEcrire.ecritUneLigne("25,26,68,67")
    fichAEcrire.ecritUneLigne("68,26,32,74")
    fichAEcrire.ecritUneLigne("74,32,31,73")
    fichAEcrire.ecritUneLigne("25,67,73,31")
    fichAEcrire.ecritUneLigne("23,24,66,65")
    fichAEcrire.ecritUneLigne("66,24,30,72")
    fichAEcrire.ecritUneLigne("71,72,30,29")
    fichAEcrire.ecritUneLigne("23,65,71,29")
    
    # Tranche
    fichAEcrire.ecritUneLigne("1,43,44,2")
    fichAEcrire.ecritUneLigne("2,44,45,3")
    fichAEcrire.ecritUneLigne("3,45,46,4")
    fichAEcrire.ecritUneLigne("4,46,47,5")
    fichAEcrire.ecritUneLigne("5,47,64,22")
    fichAEcrire.ecritUneLigne("22,64,81,39")
    fichAEcrire.ecritUneLigne("39,81,83,41")
    fichAEcrire.ecritUneLigne("41,83,84,42")
    fichAEcrire.ecritUneLigne("43,1,18,60")
    fichAEcrire.ecritUneLigne("60,18,35,77")
    fichAEcrire.ecritUneLigne("77,35,40,82")
    fichAEcrire.ecritUneLigne("82,40,42,84")
    
    # Verso
    ecritFaceVerso("1,2,7,6", fichAEcrire)
    ecritFaceVerso("2,3,8,7", fichAEcrire)
    ecritFaceVerso("3,9,8", fichAEcrire)
    ecritFaceVerso("3,4,10,9", fichAEcrire)
    ecritFaceVerso("4,5,11,10", fichAEcrire)
    ecritFaceVerso("1,6,12,18", fichAEcrire)
    ecritFaceVerso("7,8,14,13", fichAEcrire)
    ecritFaceVerso("9,10,16,15", fichAEcrire)
    ecritFaceVerso("11,5,22,17", fichAEcrire)
    ecritFaceVerso("12,13,19,18", fichAEcrire)
    ecritFaceVerso("13,14,20,19", fichAEcrire)
    ecritFaceVerso("14,15,20", fichAEcrire)
    ecritFaceVerso("15,16,21,20", fichAEcrire)
    ecritFaceVerso("16,17,22,21", fichAEcrire)
    ecritFaceVerso("18,19,24,23", fichAEcrire)
    ecritFaceVerso("19,20,25,24", fichAEcrire)
    ecritFaceVerso("20,26,25", fichAEcrire)

    # ecritFaceVerso("20,21,27,26", fichAEcrire)
    # Je remplace cette ligne pourtant valide a priori par les deux
    # suivantes, à cause d'une saleté à l'affichage sous Blender
    ecritFaceVerso("20,21,26", fichAEcrire)
    ecritFaceVerso("21,27,26", fichAEcrire)

    ecritFaceVerso("21,22,28,27", fichAEcrire)
    ecritFaceVerso("18,23,29,35", fichAEcrire)
    ecritFaceVerso("24,25,31,30", fichAEcrire)
    ecritFaceVerso("26,27,33,32", fichAEcrire)
    ecritFaceVerso("28,22,39,34", fichAEcrire)
    ecritFaceVerso("29,30,36,35", fichAEcrire)
    ecritFaceVerso("30,31,37,36", fichAEcrire)
    ecritFaceVerso("31,32,37", fichAEcrire)
    ecritFaceVerso("32,33,38,37", fichAEcrire)
    ecritFaceVerso("33,34,39,38", fichAEcrire)
    ecritFaceVerso("35,36,40", fichAEcrire)
    ecritFaceVerso("36,37,42,40", fichAEcrire)
    ecritFaceVerso("37,38,41,42", fichAEcrire)
    ecritFaceVerso("38,39,41", fichAEcrire)
    
    fichAEcrire.ecritUneLigne("</faces>")

    fichAEcrire.close()
    
def ecritFaceVerso (maLigne, fichAEcrire):
    # Tout cela paraît bien compliqué... il s'agit seulement de faire
    # en sorte que les faces verso aient elles aussi leur périmètre
    # décrit dans le sens géométrique (inverse des aiguilles d'une
    # montre)
    mesBazars = maLigne.split(',')
    pointOrigine = int(mesBazars[0]) + nbPointsSurUneFace
    maLigne2 = "%d," % (pointOrigine)
    for numTruc in range(len(mesBazars) - 1, 0, -1):
        numAugmente = int(mesBazars[numTruc]) + nbPointsSurUneFace
        maLigne2 += "%d" % (numAugmente)
        if numTruc > 1:
            maLigne2 += ","
    fichAEcrire.ecritUneLigne(maLigne2)
    
def ecritPoint (num, xrays, yankee, zulu, fichAEcrire):
    fichAEcrire.ecritUneLigne("%d,%f,%f,%f" % (num, xrays, yankee, zulu))

def ecritPointY0 (num, xrays, zulu, fichAEcrire):
    ecritPoint(num, xrays, 0.0, zulu, fichAEcrire)

def ecritPointY1 (num, xrays, zulu, fichAEcrire):
    ecritPoint(num + nbPointsSurUneFace, xrays, epaisseurMur, zulu, fichAEcrire)

# --- Fin des definitions de classes et de fonctions,
# --- debut du programme principal

epaisseurMur = 0.2
nbPointsSurUneFace = 42 # Pourquoi? Voir schéma.

principal(sys.argv)
