#! /usr/bin/env python
# -*- coding: iso-8859-1 -*-

# Ce programme traduit au format d'Alias Wavefront (.obj)
# des fichiers 3DB uniquement constitues d'assemblages de faces de couleurs
# (sans modification de coordonnees), avec eventuellement des textures.
# Cette variante-ci gere en outre la transparence (verre
# transparent non colore, "verre") et les rflexions (miroirs non colores,
# "miroir").

import sys
from os import system
from math import sin, cos, atan, pi, sqrt

# --- Definitions de classes

class Fichier:
    def __init__(self, nomFichierArg, boolEcriture):
        self.nomFichier = nomFichierArg
        if boolEcriture:
            self.fichier = open(self.nomFichier, 'w')
            self.fichier.seek(0, 0) # Se place au debut du fichier
            self.longueur = 0
        else:
            self.fichier = open(self.nomFichier, 'r')
            self.fichier.seek(0, 2) # Se place a la fin du fichier
            self.longueur = self.fichier.tell()
        self.index = 0

    def close (self):
        self.fichier.close()

    def ecritUneLigne (self, ligneAEcrire):
        for numSigne in range(len(ligneAEcrire)):
            signe = ligneAEcrire[numSigne]
            self.ecritUnOctet(signe)
        # self.ecritUnOctet(b'\n') # Ecrit sous Windows comme b'\x0D' et non comme b'\x0A'
        self.ecritUnOctet(b'\x0A') # Encore  tester sous Windows

    def ecritUnOctet (self, signe):
        self.fichier.seek(self.index, 0)
        self.fichier.write(signe)
        self.index += 1
        self.longueur += 1

    def interromptLecture (self):
        self.close()

    def litUneLigne (self):
        octetLu = ''
        ligneLue = ''
        finDeLigne = 0 # false

        while self.index < self.longueur and not finDeLigne:
            octetLu = self.litUnOctet(self.index)
            if octetLu == '\n':
                finDeLigne = 1 # true
            else:
                ligneLue += octetLu

        return ligneLue

    def litUnOctet (self, numOctet):
        self.fichier.seek(numOctet, 0)
        octet = self.fichier.read(1)
        self.index += 1
        return octet

    def reprendLecture (self):
        self.fichier = open(self.nomFichier, 'r')
        self.fichier.seek(self.index, 0) # Se place ou on s'etait arrete

    def seek (self, numOctet):
        self.fichier.seek(numOctet, 0)
        self.index = numOctet

class HueSaturationGreyscale:
    def __init__ (self, hueArg, satArg, greyArg):
        self.hue = hueArg # En degres, en principe
        self.saturation = satArg
        self.greyscale = greyArg

class Point3D:
    def __init__ (self, xraysArg, yankeeArg, zuluArg):
        self.xrays = xraysArg
        self.yankee = yankeeArg
        self.zulu = zuluArg

class PointUV:
    def __init__ (self, uniformArg, victorArg):
        self.uniform = uniformArg
        self.victor = victorArg

class RedGreenBlue:
    def __init__ (self, redArg, greenArg, blueArg, textureArg, verreArg, miroirArg):
        self.red = redArg
        self.green = greenArg
        self.blue = blueArg
        self.texture = textureArg
        self.verre = verreArg
        self.miroir = miroirArg

# --- Definitions de fonctions

def principal (argv):
    nbArgs = len(sys.argv)

    if nbArgs >= 3:
        nomFichALire = sys.argv[1]
        prefixeDesDeuxFichiers = sys.argv[2]
    else:
        print "*** Il faut donner en arg. le nom des fichiers lu et ecrit (ce dernier sans ext.)."
        sys.exit(1)

    vrcIni = litFichIni('3db_vers_alias_obj.ini')
    texturesAvecPath = vrcIni['texturesAvecPath']
    pathTextures = vrcIni['pathTextures']
    if pathTextures[len(pathTextures) - 1] != '/':
        pathTextures += '/'

    fichierSource = Fichier(nomFichALire, 0)
    fichObjAEcrire = Fichier(prefixeDesDeuxFichiers + ".obj", 1)

    nbCouleurs = 0
    mesCouleurs = []

    nbObjets = 0
    
    nbPoints = 0
    nbPointsUV = 0
    numPremierPoint = 0 # Juste pour initialiser; en C, on declarerait la variable
    numPremierPointUV = 0 # Idem
    mesPoints = [] # Une remise a zero aura lieu apres chaque balise </faces>
    mesPointsUV = [] # Idem
    
    nbNormales = 0
    mesNormales = [] # Une remise a zero aura lieu apres chaque balise </faces>

    recopieEnTeteObj(fichObjAEcrire, prefixeDesDeuxFichiers)

    baliseEnCours = "rien"
    textureEnCours = "rien"
    lignesPoints = []
    lignesPointsUV = []
    lignesFaces = []
    yAUneTexture = False # Par defaut
    while 1:
        ligneLue = fichierSource.litUneLigne()
        if ligneLue == '':
            break
        else:
            ligneLue = chomp(ligneLue)
            if ligneLue[0] == '<':
                # On a affaire a une ligne de balise
                if ligneLue[1] != '/':
                    # On a affaire a une balise ouvrante
                    baliseEnCours = ligneLue[1:len(ligneLue) - 1]
                    if baliseEnCours == "points3D":
                        numPremierPoint = nbPoints + 1
                        # Wavefront numerote a partir de 1
                        
                        nbObjets += 1
                    elif baliseEnCours == 'pointsUV':
                        numPremierPointUV = nbPointsUV + 1
                        # Wavefront numerote a partir de 1
                        lignesPointsUV = []
                else:
                    # On a affaire a une balise fermante
                    baliseEnCours = "rien"
                    baliseFermee = ligneLue[2:len(ligneLue) - 1]
                    if baliseFermee == "faces":
                        enteteObjet(fichObjAEcrire, nbObjets)
                        recopiePoints(fichObjAEcrire, mesPoints)
                        recopiePointsUV(fichObjAEcrire, mesPointsUV, mesPoints, yAUneTexture)

                        for numNormale in range(len(mesNormales)):
                            fichObjAEcrire.ecritUneLigne(mesNormales[numNormale])
                        mesNormales = []

                        numCouleurSurTroisChiffres = surNChiffres(nbCouleurs - 1, 3)
                        # nbCouleurs est minore de 1 because moi, je fais demarrer la
                        # numerotation a zero, contrairement aux habitudes de Wavefront
                        ligneAEcrire = "usemtl Color.%s" % (numCouleurSurTroisChiffres)
                        fichObjAEcrire.ecritUneLigne(ligneAEcrire)
                        fichObjAEcrire.ecritUneLigne("s off") # Je l'ecris a chaque fois...
                        #... alors que Blender ne l'ecrit qu'une fois; je pense que ce
                        # que je fais est plus clair

                        recopieFaces(fichObjAEcrire, lignesFaces)
                        lignesFaces = []

                        yAUneTexture = False
                        
                        mesPoints = []
                        mesPointsUV = []
            else:
                # On a affaire a une ligne d'instructions
                if baliseEnCours == 'points3D':
                    nbPoints += 1
                    mesBazars = ligneLue.split(',')
                    # numLu = (int) (mesBazars[0]) # Je n'en ai pas l'usage, mais c'est plus clair
                    xrays = (float) (mesBazars[1])
                    yankee = (float) (mesBazars[2])
                    zulu = (float) (mesBazars[3])
                    # Malheureusement, le format Wavefront n'utilise pas le repere
                    # "right-handed" ordinaire en Europe: y devient -z et z devient y.
                    # La peste soit des anglo-saxons et de leur isolationnisme!
                    xraysLeftHanded = xrays
                    yankeeLeftHanded = zulu
                    zuluLeftHanded = yankee * -1.0
                    monPoint = Point3D(xraysLeftHanded, yankeeLeftHanded, zuluLeftHanded)
                    mesPoints.append(monPoint)
                elif baliseEnCours == 'pointsUV':
                    nbPointsUV += 1
                    mesBazars = ligneLue.split(',')
                    # numLu = (int) (mesBazars[0]) # Je n'en ai pas l'usage, mais c'est plus clair
                    uniform = (float) (mesBazars[1])
                    victor = (float) (mesBazars[2])
                    monPointUV = PointUV(uniform, victor)
                    mesPointsUV.append(monPointUV)
                elif baliseEnCours == 'faces':
                    mesBazars = ligneLue.split(',')
                    nbPtsDansPerim = len(mesBazars)

                    # D'abord, calculons et stockons en memoire la normale
                    numPoint0 = int(mesBazars[0]) - 1 # La numerotation CBB commence a 1
                    numPoint1 = int(mesBazars[1]) - 1 # La numerotation CBB commence a 1
                    numPoint2 = int(mesBazars[2]) - 1 # La numerotation CBB commence a 1
                    monPoint0 = mesPoints[numPoint0]
                    monPoint1 = mesPoints[numPoint1]
                    monPoint2 = mesPoints[numPoint2]
                    maNormale = normaleTriangle(monPoint0, monPoint1, monPoint2)
                    ligneAEcrire = "vn %f %f %f" % (maNormale.xrays,
                                                    maNormale.yankee, maNormale.zulu)
                    mesNormales.append(ligneAEcrire) # On stockera lors de la balise </faces>
                    nbNormales += 1

                    # Ensuite, stockons en memoire le perimetre et le numero de la normale
                    ligneFace = "f "
                    for numPtPerim in range(nbPtsDansPerim):
                        numConvEnInt = int(mesBazars[numPtPerim])
                        numConvEnInt -= 1 # La numerotation CBB demarre a 1 et non a 0
                        numConvEnInt += numPremierPoint
                        if not yAUneTexture:
                            if numPtPerim == nbPtsDansPerim - 1:
                                ligneFace += "%d//%d" % (numConvEnInt, nbNormales)
                            else:
                                ligneFace += "%d//%d " % (numConvEnInt, nbNormales)
                        else:
                            if numPtPerim == nbPtsDansPerim - 1:
                                ligneFace += "%d/%d/%d" % (numConvEnInt, numConvEnInt, nbNormales)
                            else:
                                ligneFace += "%d/%d/%d " % (numConvEnInt, numConvEnInt, nbNormales)
                    lignesFaces.append(ligneFace)
                elif baliseEnCours == "couleur":
                    yAUneTexture = False
                    nbCouleurs += 1
                    if ligneLue == "verre":
                        myRgb = RedGreenBlue(0.0, 0.0, 0.0, 'no texture', True, False)
                        mesCouleurs.append(myRgb)
                    elif ligneLue == "miroir":
                        myRgb = RedGreenBlue(0.0, 0.0, 0.0, 'no texture', False, True)
                        mesCouleurs.append(myRgb)
                    else:
                        # C'est une dfinition de couleur classique
                        mesBazars = ligneLue.split(',')
                        hue = float(mesBazars[0])
                        saturation = float(mesBazars[1])
                        greyscale = float(mesBazars[2])
                        myHsg = HueSaturationGreyscale(hue, saturation, greyscale)
                        myRgb = RedGreenBlue(0.0, 0.0, 0.0, 'no texture', False, False)
                        myRgb = hsgToRgb(myRgb, myHsg)
                        # couleurEnCours = myRgb
                        mesCouleurs.append(myRgb)
                elif baliseEnCours == "texture":
                    yAUneTexture = True
                    textureEnCours = ligneLue
                    if texturesAvecPath:
                        textureEnCours = pathTextures + ligneLue
                    else:
                        textureEnCours = ligneLue
                    nbCouleurs += 1 # Une texture n'est pas vraiment une couleur, mais bon
                    myRgb = RedGreenBlue(0.0, 0.0, 0.0, textureEnCours, False, False)
                    # couleurEnCours = myRgb
                    mesCouleurs.append(myRgb)
                else:
                    pass

    fichierSource.close()
    fichObjAEcrire.close()

    fichMtlAEcrire = Fichier(prefixeDesDeuxFichiers + ".mtl", 1)
    recopieEnTeteMtl(fichMtlAEcrire, nbCouleurs)
    for numCouleur in range(nbCouleurs):
        maCouleur = mesCouleurs[numCouleur]
        nomCouleur = "Color.%s" % (surNChiffres(numCouleur, 3))
        ligneAEcrire = "newmtl %s" % (nomCouleur)
        fichMtlAEcrire.ecritUneLigne(ligneAEcrire)
        if maCouleur.verre or maCouleur.miroir:
            fichMtlAEcrire.ecritUneLigne("Ns 900.0")
            fichMtlAEcrire.ecritUneLigne("Ka 1.0 1.0 1.0")
        else:
            fichMtlAEcrire.ecritUneLigne("Ns 96.078431")
            fichMtlAEcrire.ecritUneLigne("Ka 0.333000 0.333000 0.333000")
        if maCouleur.texture == 'no texture':
            if maCouleur.verre or maCouleur.miroir:
                ligneAEcrire = "Kd 0.8 0.8 0.8"
            else:
                ligneAEcrire = "Kd %f %f %f" % (maCouleur.red, maCouleur.green, maCouleur.blue)
        else:
            ligneAEcrire = "Kd 0.85 0.85 0.0" # C'est du jaune
        fichMtlAEcrire.ecritUneLigne(ligneAEcrire)
        if maCouleur.verre or maCouleur.miroir:
            fichMtlAEcrire.ecritUneLigne("Ks 0.5 0.5 0.5")
        else:
            fichMtlAEcrire.ecritUneLigne("Ks 0.200500 0.200500 0.200500")
        fichMtlAEcrire.ecritUneLigne("Ke 0.000000 0.000000 0.000000")
        if maCouleur.miroir:
            fichMtlAEcrire.ecritUneLigne("Ni 1.45")
            fichMtlAEcrire.ecritUneLigne("d 1.000000")
            fichMtlAEcrire.ecritUneLigne("illum 3")
        elif maCouleur.verre:
            fichMtlAEcrire.ecritUneLigne("Ni 1.45")
            fichMtlAEcrire.ecritUneLigne("d 0.480938")
            fichMtlAEcrire.ecritUneLigne("illum 6")
        else:
            fichMtlAEcrire.ecritUneLigne("Ni 1.000000")
            fichMtlAEcrire.ecritUneLigne("d 1.000000")
            fichMtlAEcrire.ecritUneLigne("illum 2")
        if maCouleur.texture != 'no texture':
            fichMtlAEcrire.ecritUneLigne("map_Kd %s" % (maCouleur.texture))
        if numCouleur < nbCouleurs - 1:
            fichMtlAEcrire.ecritUneLigne("")
    fichMtlAEcrire.close()

def brightnessToGrey (myRgb, saturation, desiredGrey):
    red = myRgb.red
    green = myRgb.green
    blue = myRgb.blue

    pureColorGrey = (76.3333 / 255.0) * red + \
                    (150.3334 / 255.0) * green + \
                    (28.3333 / 255.0) * blue

    pureColorRed = red
    pureColorGreen = green
    pureColorBlue = blue

    red   = saturation * pureColorRed   + (1.0 - saturation) * pureColorGrey
    green = saturation * pureColorGreen + (1.0 - saturation) * pureColorGrey
    blue  = saturation * pureColorBlue  + (1.0 - saturation) * pureColorGrey

    modifiedColorGrey = (76.3333 / 255.0) * red + \
                        (150.3334 / 255.0) * green + \
                        (28.3333 / 255.0) * blue

    if modifiedColorGrey >= desiredGrey:
        red   *= (desiredGrey / modifiedColorGrey)
        green *= (desiredGrey / modifiedColorGrey)
        blue  *= (desiredGrey / modifiedColorGrey)
    else:
        red   = 1.0 - ((1.0 - red)   * ((1.0 - desiredGrey) / (1.0 - modifiedColorGrey)))
        green = 1.0 - ((1.0 - green) * ((1.0 - desiredGrey) / (1.0 - modifiedColorGrey)))
        blue  = 1.0 - ((1.0 - blue)  * ((1.0 - desiredGrey) / (1.0 - modifiedColorGrey)))

    myRgb.red = red
    myRgb.green = green
    myRgb.blue = blue

    return myRgb

def chomp (maChaine):
# remplace la fonction du meme nom en Perl, qui vire le ou les
# caracteres de fin de ligne
    index = 0
    maChaine2 = ''
    while index < len(maChaine):
        signe = maChaine[index]
        if signe != '\n' and signe != '\r':
            maChaine2 += signe
        index += 1
    return maChaine2

def distance3D (pointA, pointB):
    deltaX = pointB.xrays - pointA.xrays
    deltaY = pointB.yankee - pointA.yankee
    deltaZ = pointB.zulu - pointA.zulu
    distDansPlanXY = sqrt(deltaX * deltaX + deltaY * deltaY)
    distance = sqrt(distDansPlanXY * distDansPlanXY + deltaZ * deltaZ)

    return distance

def enteteObjet (fichAEcrire, nbObjets):
    nomObjet = "Shape.%s" % (surNChiffres(nbObjets - 1, 3))
    # Je minore de 1 parce que moi, je prefere commencer la numerotation a zero
    fichAEcrire.ecritUneLigne("o %s" % (nomObjet))

def hsgToRgb (myRgb, myHsg):
    hueInDegrees = myHsg.hue
    saturation = myHsg.saturation
    grey = myHsg.greyscale
    pi = 4.0 * atan(1.0)

    if grey <= 0.0:
        grey = 0.0
    if grey >= 1.0:
        grey = 1.0

    if saturation <= 0.0:
        saturation = 0.0
    if saturation >= 1.0:
        saturation = 1.0

    while hueInDegrees < 0.0:
        hueInDegrees += 360.0
    while hueInDegrees >= 360.0:
        hueInDegrees -= 360.0

    hueInRad = hueInDegrees * pi / 180.0

    hueSector = int(hueInDegrees / 60.0)
    if hueSector < 0:
        hueSector = 5
    if hueSector > 5:
        hueSector = 0

    if hueSector == 0:
        blue = 0.0
        red = 1.0
        green = (1.0 - cos(hueInRad * 3.0)) / 2.0
    elif hueSector == 1:
        blue = 0.0
        green = 1.0
        red = (1.0 - cos(hueInRad * 3.0)) / 2.0
    elif hueSector == 2:
        red = 0.0
        green = 1.0
        blue = (1.0 - cos(hueInRad * 3.0)) / 2.0
    elif hueSector == 3:
        red = 0.0
        blue = 1.0
        green = (1.0 - cos(hueInRad * 3.0)) / 2.0
    elif hueSector == 4:
        green = 0.0
        blue = 1.0
        red = (1.0 - cos(hueInRad * 3.0)) / 2.0
    else:
        green = 0.0
        red = 1.0
        blue = (1.0 - cos(hueInRad * 3.0)) / 2.0

    myRgb.red = red
    myRgb.green = green
    myRgb.blue = blue

    brightnessToGrey(myRgb, saturation, grey)

    return myRgb

def litFichIni (nomFichIni):
    vrcIni = {}
    
    # Lecture du fichier d'initialisation et stockage de toutes
    # les valeurs dans le dictionnaire, mais sous la forme de chaines
    fichierSource = open(nomFichIni, 'r')
    ligneLue = 'rien'
    while ligneLue != '':
        ligneLue = fichierSource.readline()
        if ligneLue != '':
            ligneLue2 = chomp(ligneLue)
            if yAUnSigne('=', ligneLue2):
                tableau = ligneLue2.split('=')
                cle = tableau[0].strip()
                valeur = tableau[1].strip()
                vrcIni[cle] = valeur
    fichierSource.close()
    
    # Affectation du type correct a chacune des valeurs du dictionnaire
    cles = vrcIni.keys()
    for cle in cles:
        if typeString(vrcIni[cle]) == 1:
            vrcIni[cle] = float(vrcIni[cle])
        elif typeString(vrcIni[cle]) == 2:
            vrcIni[cle] = int(vrcIni[cle])
        elif vrcIni[cle].lower() == 'true':
            vrcIni[cle] = True
        elif vrcIni[cle].lower() == 'false':
            vrcIni[cle] = False

    return vrcIni

def normaleTriangle (pointA, pointB, pointC):
    # Je confesse ne pas comprendre grand-chose a la logique de ce calcul,
    # j'ai servilement pompe sur l'article "Produit vectoriel" de Wikipedia.
    # Le fait que le format Wavefront n'utilise pas le systeme de coordonnees
    # "right-handed" habituel des Europeens (x vers la droite, y vers l'avant,
    # z vers le haut) ne semble pas influer l'utilisabilite des resultats,
    # surtout ne me demandez pas de vous l'expliquer.
    
    u1 = pointB.xrays - pointA.xrays
    u2 = pointB.yankee - pointA.yankee
    u3 = pointB.zulu - pointA.zulu

    v1 = pointC.xrays - pointA.xrays
    v2 = pointC.yankee - pointA.yankee
    v3 = pointC.zulu - pointA.zulu

    xNormale = u2 * v3 - u3 * v2
    yNormale = u3 * v1 - u1 * v3
    zNormale = u1 * v2 - u2 * v1
    pointNormaleBrut = Point3D(xNormale, yNormale, zNormale)

    # Pour faire les choses plus proprement, on met ce pointNormale
    # a une distance de 1.0
    pointO = Point3D(0.0, 0.0, 0.0)
    distance = distance3D(pointO, pointNormaleBrut)
    if distance != 0:
        pointNormale = Point3D(pointNormaleBrut.xrays / distance,
                               pointNormaleBrut.yankee / distance,
                               pointNormaleBrut.zulu / distance)
    else:
        # A mon avis, ca ne devrait pas arriver, mais ca arrive quand meme
        # trop souvent pour qu'on ne le prevoie pas... Pas la peine
        # de planter a cause d'une face non affichable!
        print("Warning: une face non visible.")
        pointNormale = Point3D(0.0, 0.0, 1.0)

    return pointNormale

def recopieEnTeteMtl (fichAEcrire, nbCouleurs):
    fichAEcrire.ecritUneLigne("# Jlama MTL File")
    fichAEcrire.ecritUneLigne("# Material Count: %d" % (nbCouleurs))
    fichAEcrire.ecritUneLigne("")

def recopieEnTeteObj (fichAEcrire, prefixeDesDeuxFichiers):
    fichAEcrire.ecritUneLigne("# Jlama OBJ File")
    fichAEcrire.ecritUneLigne("mtllib %s.mtl" % (prefixeDesDeuxFichiers))

def recopieFaces (fichAEcrire, lignesFaces):
    for numFace in range(len(lignesFaces)):
        fichAEcrire.ecritUneLigne(lignesFaces[numFace])

def recopiePoints (fichAEcrire, mesPoints):
    for numPoint in range(len(mesPoints)):
        monPoint = mesPoints[numPoint]
        lignePoint = "v %f %f %f" % (monPoint.xrays, monPoint.yankee, monPoint.zulu)
        fichAEcrire.ecritUneLigne(lignePoint)

def recopiePointsUV (fichAEcrire, mesPointsUV, mesPoints, yAUneTexture):
    if yAUneTexture:
        for numPointUV in range(len(mesPointsUV)):
            monPointUV = mesPointsUV[numPointUV]
            lignePointUV = "vt %f %f" % (monPointUV.uniform, monPointUV.victor)
            fichAEcrire.ecritUneLigne(lignePointUV)
    else:
        # Je fais comme ce sagouin de Blender parce qu'en effet c'est le plus
        # simple: quand il n'y a pas de coordonnees UV, j'en mets quand meme...
        # parce que ca evite d'avoir a retravailler les index
        for numPoint in range(len(mesPoints)):
            monPoint = mesPoints[numPoint]
            lignePointUV = "vt 0.0 0.0"
            fichAEcrire.ecritUneLigne(lignePointUV)

def surNChiffres (num, nbChiffres):
    chaine = "%d" % (num)
    while len(chaine) < nbChiffres:
        chaine = '0' + chaine

    return chaine

def typeString (maChaine):
# Python etant type, il faut pouvoir determiner le type
# d'une variable enregistree en Ascii dans le fichier
# d'initialisation. Cette routine reconnait les nombres,
# distingue entre entiers et nombres a point decimal, et
# considere tout le reste comme des chaines
    index = 0
    boolNumerique = 1 # true
    while index < len(maChaine):
        signe = maChaine[index]
        if not (signe == '.' or signe == '-' or \
                (signe >= '0' and signe <= '9')):
            boolNumerique = 0 # false
        index += 1

    boolEntier = 1 # true
    if boolNumerique:
        index = 0
        while index < len(maChaine):
            signe = maChaine[index]
            if signe == '.':
                boolEntier = 0 # false
            index += 1

    typeDeChaine = boolNumerique
    if boolNumerique and boolEntier:
        typeDeChaine = 2

    return typeDeChaine # 0 = str, 1 = float, 2 = int

def yAUnSigne (signeAChercher, maChaine):
# S'assure de la presence d'un signe precis dans une ligne.
# Sert notamment a reperer le signe = dans les lignes d'un
# fichier d'initialisation
    boolYAUnSigne = 0 # false
    index = 0
    while index < len(maChaine):
        signe = maChaine[index]
        if signe == signeAChercher:
            boolYAUnSigne = 1 # true
        index += 1
    return boolYAUnSigne

# --- Fin des definitions de classes et de fonctions,
# --- debut du programme principal

principal(sys.argv)
